const express = require('express')
const schedulerRepo = require('../repository/scheduler')

module.exports = function (db, treant) {
  const scheduler = schedulerRepo(db)
  var router = express.Router()

  router.route('/')
  .post(treant.check({getAcc: true}), function (req, res) {
    var item = req.body
    item.author = {
      id: req.account._id.toString(),
      name: req.account.name
    }

    scheduler.insert(item, function (err) {
      if (err) return res.status(500).json(err)

      res.header({location: item._id}).status(204).send()
    })
  })
  .get(function (req, res) {
    var selector = {
      pageIndex: Number(req.query.pageIndex),
      pageSize: Number(req.query.pageSize)
    }
    scheduler.search(req.query.keyword, selector, function (error, items) {
      if (error) return res.status(500).json(error)
      if (items.length < 1) {
        return res.status(404).json({message: 'not found scheduler'})
      }

      res.json(items)
    })
  })

  router.route('/:id')
  .get(function (req, res) {
    scheduler.findById(req.params.id, function (err, item) {
      if (err) return res.json(err)

      if (!item) return res.status(400).json({message: 'not found scheduler'})

      res.json(item)
    })
  })
  .delete(treant.check({getAcc: true}), function (req, res) {
    var accId = req.account._id

    scheduler.remove(req.param.id, accId, function (err, result) {
      if (err || result.result.n === 0) {
        return res.status(400).json({
          message: 'can not remove',
          data: err
        })
      }

      res.json({message: 'done'})
    })
  })

  router.route('/my')
  .get(treant.check({getAcc: true}), function (req, res) {
    var selector = {
      pageIndex: Number(req.query.pageIndex),
      pageSize: Number(req.query.pageSize)
    }
    var options = {
      author: req.account._id,
      keyword: req.query.keyword,
      schedulers: req.account.schedulers
    }

    scheduler.searchAuthor(options, selector, function (err, items) {
      if (err) return res.status(500).json(err)
      if (items.length < 1) {
        return res.status(404).json({message: 'not found scheduler'})
      }

      res.json(items)
    })
  })

  return router
}
