const imjv = require('is-my-json-valid')

const validate = imjv({
  type: 'object',
  require: true,
  properties: {
    name: {
      type: 'string',
      require: true,
      pattern: '^[a-zA-Z0-9 .-]{2,32}$'
    },
    tags: {
      type: 'array',
      items: {
        type: 'string',
        pattern: '^[a-z0-9]{2,8}$'
      }
    },
    tasks: {
      type: 'array',
      require: true,
      items: {
        type: 'object',
        properties: {
          start: {
            require: true,
            type: 'string',
            'pattern': '^([0-1][0-9]|[2][0-3]):[0-5][0-9]$'
          },
          action: {
            require: true,
            type: 'string',
            pattern: '^[a-zA-Z0-9 .-]{1,16}$'
          }
        }
      }
    },
    notes: {
      type: 'string',
      pattern: '^.{0,1024}$'
    }
  }
})

function verifyItems(items) {
  if (typeof(items) !== 'object') return false

  if (Array.isArray(items)) {
    for (var i = 0; i < items.length; i++) {
      if (!validate(items[i])) {
        console.log(validate.errors);
        return false
      }
    }
  } else {
    return validate(items)
  }

  return true
}

module.exports = {
  verify: verifyItems,
  error: function () {
      return validate.errors
  }
}
