const jsonfile = require('jsonfile')
const async = require('async')
const path = require('path')
const SchedulerRepository = require('../repository/scheduler')

const randstr = require('./randstr')

// import from file to database
// file in json format
// params
//  - opts: object contains options
//    - dir <string>: path to data directory. each file in  directory is one
//      scheduler
//    - db <mongo>: instance of mongodb
//  - callback <function>: function will call
//    params
//     - error <object>: error during process
module.exports = function (opts, callback) {
  const numGen = 512
  const scheduler = new SchedulerRepository(opts.db)
  var calls = []
  var items = []

  // read meaning scheduler from file
  calls.push(function (callback) {
    var file = path.join(opts.assetDir, 'scheduler-001.json')
    jsonfile.readFile(file, function (err, item) {
      if (err) return callback(err)

      items.push(item)
      callback()
    })
  })

  // generate random scheduler
  // it is not meaning to use. it use to fill database for testing
  for (var i = 0; i < numGen; i++) {
    items.push({
      name: randstr(16),
      author: {id: '57da7b1e56cbb6332008e064', name: 'kevin'},
      notes: randstr(1024),
      tags: ['home', 'relax', 'holiday'],
      tasks: [
        {start: '00:00', action: randstr(16)},
        {start: '03:00', action: randstr(16)},
        {start: '06:00', action: randstr(16)},
        {start: '09:00', action: randstr(16)},
        {start: '12:00', action: randstr(16)},
        {start: '15:00', action: randstr(16)},
        {start: '18:00', action: randstr(16)},
        {start: '21:00', action: randstr(16)}
      ]
    })
  }

  // push to database
  calls.push(function (callback) {
    scheduler.insert(items, callback)
  })

  // execute
  async.series(calls, function (err) {
    callback(err)
  })
}
