const mongodb = require('mongodb')

const ObjectID = mongodb.ObjectID

// account repository
// using to query account in database and other oauth provider
// params
//  - db <connection>: mongoose database connection
//  - oauth <object>: contains information about oauth provider
// return instance of account repository
function AccountRepository (dbc) {
  this._dbc = dbc
  this._collection = dbc.collection('account')
  this._scheduler = dbc.collection('scheduler')
}

// shortcut to account repository prototype
// using to access to repository prototype more quickly
var prototype = AccountRepository.prototype

// add scheduler which user using
// params
//  - subject <string>: identity of account in provider
//  - schedulerId <string>: scheduler identity
//  - callback <function>: function will call
//    params
//     - error <object>: error during process
prototype.useScheduler = function (account, schedulerId, callback) {
  const self = this

  // verify that scheduler is exists
  self._scheduler.find(schedulerId, function (err, scheduler) {
    if (err) return callback(err)

    // only add scheduler if it is not early exist
    if (account.schedulers.indexOf(schedulerId) < 0) {
      account.schedulers.push(schedulerId)
    }
    account.scheduler = schedulerId

    saveAccount(account, callback)
  })

  function saveAccount (account, callback) {
    var patch = {
      $set: {
        scheduler: account.scheduler,
        schedulers: account.schedulers
      }
    }
    self._account.update({_id: ObjectID(account.id)}, patch, function (err) {
      callback(err)
    })
  }
}

// update user language
// params
//  - subject <object>: identity of account in provider
//  - language <string>: language code, same as 'eng'
//  - callback <function>: function will called
//    params
//       - error <object>: error return by process
prototype.useLanguage = function (id, language, callback) {
  var path = {$set: {language: language}}

  this._collection.update({_id: ObjectID(id)}, path, function (err) {
    callback(err)
  })
}

// export module
module.exports = AccountRepository
