const mongodb = require('mongodb')

const schedulerModel = require('../model/scheduler')

const ObjectID = mongodb.ObjectID

function SchedulerRepo (dbc) {
  this._dbc = dbc
  this._collection = dbc.collection('scheduler')
}

// shortcut to repository prototype
// using to access to repositoy prototype more quickly
const prototype = SchedulerRepo.prototype

prototype.insert = function (items, callback) {
  if (!schedulerModel.verify(items)) return callback(schedulerModel.error())

  this._collection.insert(items, function (err) {
    callback(err)
  })
}

// find item by id, id coresponse with _id field
// params
//  - id <string>: identity of item
//  - callback <function>: function will call
//    params
//       - error <object>: error return by process
//       - item <array>: list of item has found
prototype.findById = function (id, callback) {
  this._collection.findOne({_id: ObjectID(id)}, function (error, item) {
    callback(error, item)
  })
}

// remove
prototype.remove = function (id, author, callback) {
  this._collection.findOne({_id: ObjectID(id)}, function (err, item) {
    if (err || !item) return callback(err, item)
    if (item.author.id !== author) {
      return callback({message: 'can not remove scheduler of other user'})
    }

    this._collection.remove({_id: ObjectID(id)}, function (err, res) {
      callback(err, res)
    })
  })
}

// search item by keyword, keyword will match with fields below
//  - name
//  - element of tags
// params
//  - keyword <string>: keyword for search
//  - callback <function>: function will called
//    params
//     - error <object>: error return by process
//     - items <array>: list of items matched by search
prototype.search = function (keyword, selector, callback) {
  var skip = (selector.pageIndex - 1) * selector.pageSize
  var options = { skip: skip, limit: selector.pageSize }
  var filter = {}
  if (keyword !== '*') {
    filter['$or'] = [
      { name: { $regex: new RegExp(keyword, 'gi') } },
      { tags: { $elemMatch: { $regex: new RegExp(keyword, 'gi') } } }
    ]
  }
  var fields = {notes: 0, tasks: 0, tags: 0}

  this._collection.find(filter, fields, options)
  .toArray(function (err, items) {
    callback(err, items)
  })
}

// same as search, but match with account identity
// params
//  - itemIds: item identity which user use of other user
prototype.searchAuthor = function (options, selector, callback) {
  var skip = (selector.pageIndex - 1) * selector.pageSize
  var queryOpts = {skip: skip, limit: selector.pageSize}
  var fields = {notes: 0, tasks: 0, tags: 0}
  if (!options.ids) options.ids = []
  var filter = {$and: [ { $or: [
    {author: options.author},
    {_id: { $in: options.ids }}
  ]}]}

  if (options.keyword !== '*') {
    filter['$and'].push({$or: [
      { name: { $regex: new RegExp(options.keyword, 'gi') } },
      { tags: { $elemMatch: { $regex: new RegExp(options.keyword, 'gi') } } }
    ]})
  }

  this._collection.find(filter, fields, queryOpts)
  .toArray(function (err, items) {
    callback(err, items)
  })
}

// define module
module.exports = function (dbc) {
  return new SchedulerRepo(dbc)
}
